local MAX_TABS = 4;

local LS = AceLibrary("AceLocale-2.2"):new("Historian-Shared");
local f = AceLibrary("HistorianFormatting-1.0");
local abacus = AceLibrary("Abacus-2.0");

HISTORIAN_FRAME_HABITS = LS["STAT_CATEGORY_HABITS"];
HISTORIAN_FRAME_ACHIEVEMENTS = LS["STAT_CATEGORY_ACHIEVEMENTS"];
HISTORIAN_FRAME_DISAPPOINTMENTS = LS["STAT_CATEGORY_DISAPPOINTMENTS"];
HISTORIAN_FRAME_FORTUNE = LS["STAT_CATEGORY_FORTUNE"];

local leftText = "";
local rightText = "";
local leftCell = nil;
local rightCell = nil;

-- *************************************
-- * Utility functions for data display
-- *************************************
local function AddLineLeft(text)
	leftText = leftText .. text .. "\n";
end;

local function AddLineRight(text)
	rightText = rightText .. text .. "\n";
end;

local function AddBlankLine()
	AddLineLeft("");
	AddLineRight("");
end;

local function AddNumberStat(label, valueLabel, number)
	AddLineLeft(label);

	if (number == nil) or (number < 1) then
		return AddLineRight(LS["NOT_AVAILABLE"]);
	end;

	if (valueLabel ~= nil) then
		AddLineRight(string.format(valueLabel, f:IntToStr(number)));
	else
		AddLineRight(f:IntToStr(number));
	end;
end;

local function AddNameAndNumberStat(label, valueLabel, name, number)
	AddLineLeft(label);

	if (number == nil) or (number < 1) then
		return AddLineRight(LS["NOT_AVAILABLE"]);
	end;

	AddLineRight(string.format(valueLabel, name, f:IntToStr(number)));
end;


local function ClearCurrentText()
	leftText = "";
	rightText = "";
end;


local function SetNewText()
	leftCell:SetText(leftText);
	rightCell:SetText(rightText);
end;
	
	

HistorianGUI = {};


-- *************************************
-- * OnLoad
-- *************************************
function HistorianGUI:OnLoad()
	PanelTemplates_SetNumTabs(HistorianTabContainer, MAX_TABS);
	PanelTemplates_SetTab(HistorianTabContainer, 1);
	self.currentTab = 1;
	
	-- Enable closing of the window using ESC
	tinsert(UISpecialFrames,"HistorianTabContainer");
end;


-- *************************************
-- * OnShow
-- *************************************
function HistorianGUI:OnShow()
	HistorianTabContainer.IsShowing = true;
	PlaySound("UChatScrollButton");
	
	-- Reload the current tab
	self:ShowTab(self.currentTab);
end;


-- *************************************
-- * OnHide
-- *************************************
function HistorianGUI:OnHide()
	HistorianTabContainer.IsShowing = false;
	PlaySound("UChatScrollButton");
end;


-- *************************************
-- * OnHeaderMouseDown
-- *************************************
function HistorianGUI:OnHeaderMouseDown()
	local frame = this:GetParent();
	
	if (arg1 == "LeftButton") then
		frame:StartMoving();
	elseif (arg1 == "RightButton") then
		frame:ClearAllPoints();
		frame:SetPoint("CENTER", "UIParent", "CENTER");
	end
end;


-- *************************************
-- * OnHeaderMouseUp
-- *************************************
function HistorianGUI:OnHeaderMouseUp()
	local frame = this:GetParent();
	frame:StopMovingOrSizing();
end;


-- *************************************
-- * ShowTab
-- *************************************
function HistorianGUI:ShowTab(tabID)
	PanelTemplates_SetTab(HistorianTabContainer, tabID);

	for i = 1, MAX_TABS do
		local tab = getglobal("HistorianTabPage" .. i);
		if (tab ~= nil) then
			if (i == tabID) then
				tab:Show();
			else
				tab:Hide();
			end;
		end;
	end;
	
	self.currentTab = tabID;
	self:ShowContent(tabID);
end;


-- *************************************
-- * OnTabClicked
-- *************************************
function HistorianGUI:OnTabClicked(tabID)
	self:ShowTab(tabID);
end;



-- *************************************
-- * ShowContent
-- *************************************
function HistorianGUI:ShowContent(tabID)
	-- Get the two labels to update
	leftCell = getglobal("HistorianTabPage" .. tostring(tabID) .. "ChildFrameLeftCell");
	rightCell = getglobal("HistorianTabPage" .. tostring(tabID) .. "ChildFrameRightCell");
	
	if (leftCell == nil) or (rightCell == nil) then
		Historian:Print("Failed to retrieve the left and right cells for tab #" .. tostring(tabID));
		return;
	end;

	ClearCurrentText();
	
	if (tabID == 1) then
		self:ShowHabits();
	elseif (tabID == 2) then
		self:ShowAchievements();
	elseif (tabID == 3) then
		self:ShowDisappointments();
	elseif (tabID == 4) then
		self:ShowFortune();
	end;
	
	SetNewText();
end;


-- *************************************
-- * ShowHabits
-- *************************************
function HistorianGUI:ShowHabits(leftCell, rightCell)
	local data = Historian:GetHabitStats();
	
	AddNameAndNumberStat(LS["STAT_MOST_FOLLOWED"], LS["STAT_MOST_FOLLOWED_VALUE"], data.MostFollowed.Name, data.MostFollowed.Count);
	AddNameAndNumberStat(LS["STAT_FAVORITE_PREY"], LS["STAT_FAVORITE_PREY_VALUE"], data.FavoritePrey.Name, data.FavoritePrey.Count);
	AddNameAndNumberStat(LS["STAT_FAVORITE_SPELL"], LS["STAT_FAVORITE_SPELL_VALUE"], data.FavoriteSpell.Name, data.FavoriteSpell.Count);
	AddNumberStat(LS["STAT_TOTAL_SPELLS"], nil, data.TotalSpellsCast);
	AddBlankLine();
	
	AddNumberStat(LS["STAT_FOOD"], LS["STAT_FOOD_VALUE"], data.Food);
	AddNumberStat(LS["STAT_DRINK"], LS["STAT_DRINK_VALUE"], data.Drink);
	AddBlankLine();

	AddNumberStat(LS["STAT_TIPSY"], LS["STAT_TIPSY_VALUE"], data.Tipsy);
	AddNumberStat(LS["STAT_DRUNK"], LS["STAT_DRUNK_VALUE"], data.Drunk);
	AddNumberStat(LS["STAT_SMASHED"], LS["STAT_SMASHED_VALUE"], data.Smashed);
	AddBlankLine();

	AddNumberStat(LS["STAT_JUMPS"], nil, data.Jumps);
end;


-- *************************************
-- * ShowAchievements
-- *************************************
function HistorianGUI:ShowAchievements()
	local data = Historian:GetAchievementStats();
	
	AddNumberStat(LS["STAT_QUESTS_COMPLETED"], nil, data.QuestsCompleted);
	AddNumberStat(LS["STAT_RECIPES_LEARNED"], nil, data.RecipesLearned);
	AddNumberStat(LS["STAT_AREAS_DISCOVERED"], nil, data.AreasDiscovered);
	AddNumberStat(LS["STAT_TOTAL_KILLS"], nil, data.TotalKills);
end;



-- *************************************
-- * ShowDisappointments
-- *************************************
function HistorianGUI:ShowDisappointments()
	local data = Historian:GetDisappointmentStats();
	
	local function AddEnvDamageStat(label, times, totalDamage)
		AddLineLeft(label);

		if (times == nil) or (times < 1) then
			return AddLineRight(LS["NOT_AVAILABLE"]);
		end;

		local timesText = f:IntToStr(times);
		local damageText = f:IntToStr(totalDamage);

		if (times == 1) then
			AddLineRight(string.format(LS["STAT_ENV_VALUE_SINGLE"], damageText));
		else
			AddLineRight(string.format(LS["STAT_ENV_VALUE"], timesText, damageText));
		end;
	end;
	
	AddEnvDamageStat(LS["STAT_ENV_DROWNING"], data.Drowning.Times, data.Drowning.TotalDamage);
	AddEnvDamageStat(LS["STAT_ENV_FATIGUE"], data.Fatique.Times, data.Fatique.TotalDamage);
	AddEnvDamageStat(LS["STAT_ENV_FALL"], data.Falling.Times, data.Falling.TotalDamage);
	AddEnvDamageStat(LS["STAT_ENV_FIRE"], data.Fire.Times, data.Fire.TotalDamage);
	AddEnvDamageStat(LS["STAT_ENV_LAVA"], data.Lava.Times, data.Lava.TotalDamage);
	AddEnvDamageStat(LS["STAT_ENV_SLIME"], data.Slime.Times, data.Slime.TotalDamage);

	AddBlankLine();
	AddNumberStat(LS["STAT_DEATHS"], nil, data.TotalDeaths);
end;


-- *************************************
-- * ShowFortune
-- *************************************
function HistorianGUI:ShowFortune()
	local data = Historian:GetFortuneStats();

	AddLineLeft(LS["STAT_MAX_MONEY"]);
	AddLineRight(abacus:FormatMoneyFull(data.MostMoneyEver, true));
	AddBlankLine();
	
	AddNumberStat(LS["STAT_LOOTED_COMMON"], nil, data.LootedCommon);
	AddNumberStat(LS["STAT_LOOTED_UNCOMMON"], nil, data.LootedUncommon);
	AddNumberStat(LS["STAT_LOOTED_RARE"], nil, data.LootedRare);
	AddNumberStat(LS["STAT_LOOTED_EPIC"], nil, data.LootedEpic);
	AddNumberStat(LS["STAT_LOOTED_LEGENDARY"], nil, data.LootedLegendary);
	AddNumberStat(LS["STAT_LOOTED_ARTIFACT"], nil, data.LootedArtifact);
end;